# Form implementation generated from reading ui file '/home/kovid/work/calibre/src/calibre/gui2/preferences/look_feel.ui'
#
# Created by: PyQt6 UI code generator 6.7.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(1096, 791)
        self.gridLayout_2 = QtWidgets.QGridLayout(Form)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget = ScrollingTabWidget(parent=Form)
        self.tabWidget.setObjectName("tabWidget")
        self.main_interface_tab = QtWidgets.QWidget()
        self.main_interface_tab.setObjectName("main_interface_tab")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.main_interface_tab)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label_widget_style = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.label_widget_style.setObjectName("label_widget_style")
        self.gridLayout_7.addWidget(self.label_widget_style, 0, 0, 1, 1)
        self.opt_ui_style = QtWidgets.QComboBox(parent=self.main_interface_tab)
        self.opt_ui_style.setObjectName("opt_ui_style")
        self.gridLayout_7.addWidget(self.opt_ui_style, 0, 1, 1, 1)
        self.icon_theme = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.icon_theme.setText("")
        self.icon_theme.setObjectName("icon_theme")
        self.gridLayout_7.addWidget(self.icon_theme, 1, 0, 1, 1)
        self.icon_theme_button = QtWidgets.QPushButton(parent=self.main_interface_tab)
        self.icon_theme_button.setObjectName("icon_theme_button")
        self.gridLayout_7.addWidget(self.icon_theme_button, 1, 1, 1, 1)
        self.opt_color_palette_label = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.opt_color_palette_label.setObjectName("opt_color_palette_label")
        self.gridLayout_7.addWidget(self.opt_color_palette_label, 2, 0, 1, 1)
        self.button_adjust_colors = QtWidgets.QPushButton(parent=self.main_interface_tab)
        self.button_adjust_colors.setObjectName("button_adjust_colors")
        self.gridLayout_7.addWidget(self.button_adjust_colors, 2, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.label_17.setObjectName("label_17")
        self.gridLayout_7.addWidget(self.label_17, 3, 0, 1, 1)
        self.opt_gui_layout = QtWidgets.QComboBox(parent=self.main_interface_tab)
        self.opt_gui_layout.setMaximumSize(QtCore.QSize(250, 16777215))
        self.opt_gui_layout.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_gui_layout.setMinimumContentsLength(20)
        self.opt_gui_layout.setObjectName("opt_gui_layout")
        self.gridLayout_7.addWidget(self.opt_gui_layout, 3, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.label_7.setObjectName("label_7")
        self.gridLayout_7.addWidget(self.label_7, 4, 0, 1, 1)
        self.opt_language = QtWidgets.QComboBox(parent=self.main_interface_tab)
        self.opt_language.setSizeAdjustPolicy(QtWidgets.QComboBox.SizeAdjustPolicy.AdjustToMinimumContentsLengthWithIcon)
        self.opt_language.setMinimumContentsLength(20)
        self.opt_language.setObjectName("opt_language")
        self.gridLayout_7.addWidget(self.opt_language, 4, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.label_6.setObjectName("label_6")
        self.gridLayout_7.addWidget(self.label_6, 5, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.opt_cover_corner_radius = QtWidgets.QSpinBox(parent=self.main_interface_tab)
        self.opt_cover_corner_radius.setObjectName("opt_cover_corner_radius")
        self.horizontalLayout_3.addWidget(self.opt_cover_corner_radius)
        self.opt_cover_corner_radius_unit = QtWidgets.QComboBox(parent=self.main_interface_tab)
        self.opt_cover_corner_radius_unit.setObjectName("opt_cover_corner_radius_unit")
        self.horizontalLayout_3.addWidget(self.opt_cover_corner_radius_unit)
        self.gridLayout_7.addLayout(self.horizontalLayout_3, 5, 1, 1, 1)
        self.opt_systray_icon = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_systray_icon.setObjectName("opt_systray_icon")
        self.gridLayout_7.addWidget(self.opt_systray_icon, 6, 0, 1, 1)
        self.opt_disable_animations = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_disable_animations.setObjectName("opt_disable_animations")
        self.gridLayout_7.addWidget(self.opt_disable_animations, 6, 1, 1, 1)
        self.opt_disable_tray_notification = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_disable_tray_notification.setObjectName("opt_disable_tray_notification")
        self.gridLayout_7.addWidget(self.opt_disable_tray_notification, 7, 0, 1, 1)
        self.opt_show_splash_screen = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_show_splash_screen.setObjectName("opt_show_splash_screen")
        self.gridLayout_7.addWidget(self.opt_show_splash_screen, 7, 1, 1, 1)
        self.opt_book_list_tooltips = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_book_list_tooltips.setObjectName("opt_book_list_tooltips")
        self.gridLayout_7.addWidget(self.opt_book_list_tooltips, 8, 0, 1, 1)
        self.opt_show_layout_buttons = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_show_layout_buttons.setObjectName("opt_show_layout_buttons")
        self.gridLayout_7.addWidget(self.opt_show_layout_buttons, 8, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        self.font_display = QtWidgets.QLineEdit(parent=self.main_interface_tab)
        self.font_display.setReadOnly(True)
        self.font_display.setObjectName("font_display")
        self.horizontalLayout.addWidget(self.font_display)
        self.gridLayout_7.addLayout(self.horizontalLayout, 9, 0, 1, 1)
        self.change_font_button = QtWidgets.QPushButton(parent=self.main_interface_tab)
        self.change_font_button.setObjectName("change_font_button")
        self.gridLayout_7.addWidget(self.change_font_button, 9, 1, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(parent=self.main_interface_tab)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.opt_toolbar_icon_size = QtWidgets.QComboBox(parent=self.groupBox_2)
        self.opt_toolbar_icon_size.setObjectName("opt_toolbar_icon_size")
        self.gridLayout_8.addWidget(self.opt_toolbar_icon_size, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_5.setObjectName("label_5")
        self.gridLayout_8.addWidget(self.label_5, 0, 0, 1, 1)
        self.opt_toolbar_text = QtWidgets.QComboBox(parent=self.groupBox_2)
        self.opt_toolbar_text.setObjectName("opt_toolbar_text")
        self.gridLayout_8.addWidget(self.opt_toolbar_text, 1, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.gridLayout_8.addWidget(self.label_8, 1, 0, 1, 1)
        self.opt_wrap_toolbar_text = QtWidgets.QCheckBox(parent=self.groupBox_2)
        self.opt_wrap_toolbar_text.setObjectName("opt_wrap_toolbar_text")
        self.gridLayout_8.addWidget(self.opt_wrap_toolbar_text, 2, 0, 1, 2)
        self.gridLayout_7.addWidget(self.groupBox_2, 10, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(parent=self.main_interface_tab)
        self.label_4.setObjectName("label_4")
        self.gridLayout_7.addWidget(self.label_4, 11, 0, 1, 1)
        self.opt_book_list_extra_row_spacing = QtWidgets.QSpinBox(parent=self.main_interface_tab)
        self.opt_book_list_extra_row_spacing.setMinimum(-20)
        self.opt_book_list_extra_row_spacing.setObjectName("opt_book_list_extra_row_spacing")
        self.gridLayout_7.addWidget(self.opt_book_list_extra_row_spacing, 11, 1, 1, 1)
        self.opt_row_numbers_in_book_list = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_row_numbers_in_book_list.setObjectName("opt_row_numbers_in_book_list")
        self.gridLayout_7.addWidget(self.opt_row_numbers_in_book_list, 12, 0, 1, 1)
        self.opt_booklist_grid = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_booklist_grid.setObjectName("opt_booklist_grid")
        self.gridLayout_7.addWidget(self.opt_booklist_grid, 13, 0, 1, 1)
        self.opt_dnd_merge = QtWidgets.QCheckBox(parent=self.main_interface_tab)
        self.opt_dnd_merge.setObjectName("opt_dnd_merge")
        self.gridLayout_7.addWidget(self.opt_dnd_merge, 14, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_7.addItem(spacerItem, 15, 0, 1, 1)
        icon = QtGui.QIcon.ic("lt.png")
        self.tabWidget.addTab(self.main_interface_tab, icon, "")
        self.cover_grid_tab = QtWidgets.QTabWidget()
        self.cover_grid_tab.setObjectName("cover_grid_tab")
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.formLayout_2 = QtWidgets.QFormLayout(self.tab_5)
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_19 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_19.setWordWrap(True)
        self.label_19.setObjectName("label_19")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.label_19)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        self.formLayout_2.setItem(3, QtWidgets.QFormLayout.ItemRole.FieldRole, spacerItem1)
        self.label_14 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_14.setObjectName("label_14")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_14)
        self.opt_field_under_covers_in_grid = QtWidgets.QComboBox(parent=self.tab_5)
        self.opt_field_under_covers_in_grid.setObjectName("opt_field_under_covers_in_grid")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_field_under_covers_in_grid)
        self.label_16 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_16.setObjectName("label_16")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_16)
        self.opt_cover_grid_spacing = QtWidgets.QDoubleSpinBox(parent=self.tab_5)
        self.opt_cover_grid_spacing.setDecimals(2)
        self.opt_cover_grid_spacing.setSingleStep(0.1)
        self.opt_cover_grid_spacing.setObjectName("opt_cover_grid_spacing")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_cover_grid_spacing)
        self.cg_background_box = QtWidgets.QGroupBox(parent=self.tab_5)
        self.cg_background_box.setObjectName("cg_background_box")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.cg_background_box)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.cg_background_box)
        self.groupBox_4 = QtWidgets.QGroupBox(parent=self.tab_5)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.cover_grid_larger_cover = QtWidgets.QPushButton(parent=self.groupBox_4)
        icon1 = QtGui.QIcon.ic("plus.png")
        self.cover_grid_larger_cover.setIcon(icon1)
        self.cover_grid_larger_cover.setObjectName("cover_grid_larger_cover")
        self.gridLayout_4.addWidget(self.cover_grid_larger_cover, 1, 2, 1, 1)
        self.cover_grid_smaller_cover = QtWidgets.QPushButton(parent=self.groupBox_4)
        icon2 = QtGui.QIcon.ic("minus.png")
        self.cover_grid_smaller_cover.setIcon(icon2)
        self.cover_grid_smaller_cover.setObjectName("cover_grid_smaller_cover")
        self.gridLayout_4.addWidget(self.cover_grid_smaller_cover, 2, 2, 1, 1)
        self.label_12 = QtWidgets.QLabel(parent=self.groupBox_4)
        self.label_12.setObjectName("label_12")
        self.gridLayout_4.addWidget(self.label_12, 2, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(parent=self.groupBox_4)
        self.label_11.setObjectName("label_11")
        self.gridLayout_4.addWidget(self.label_11, 1, 0, 1, 1)
        self.opt_cover_grid_height = QtWidgets.QDoubleSpinBox(parent=self.groupBox_4)
        self.opt_cover_grid_height.setDecimals(1)
        self.opt_cover_grid_height.setObjectName("opt_cover_grid_height")
        self.gridLayout_4.addWidget(self.opt_cover_grid_height, 2, 1, 1, 1)
        self.cover_grid_reset_size = QtWidgets.QPushButton(parent=self.groupBox_4)
        self.cover_grid_reset_size.setObjectName("cover_grid_reset_size")
        self.gridLayout_4.addWidget(self.cover_grid_reset_size, 1, 3, 2, 1)
        self.cover_grid_aspect_ratio = QtWidgets.QLabel(parent=self.groupBox_4)
        self.cover_grid_aspect_ratio.setText("")
        self.cover_grid_aspect_ratio.setObjectName("cover_grid_aspect_ratio")
        self.gridLayout_4.addWidget(self.cover_grid_aspect_ratio, 3, 0, 1, 5)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_4.addItem(spacerItem2, 1, 4, 1, 1)
        self.opt_cover_grid_width = QtWidgets.QDoubleSpinBox(parent=self.groupBox_4)
        self.opt_cover_grid_width.setDecimals(1)
        self.opt_cover_grid_width.setObjectName("opt_cover_grid_width")
        self.gridLayout_4.addWidget(self.opt_cover_grid_width, 1, 1, 1, 1)
        self.cover_size_label = QtWidgets.QLabel(parent=self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cover_size_label.sizePolicy().hasHeightForWidth())
        self.cover_size_label.setSizePolicy(sizePolicy)
        self.cover_size_label.setWordWrap(True)
        self.cover_size_label.setObjectName("cover_size_label")
        self.gridLayout_4.addWidget(self.cover_size_label, 0, 0, 1, 5)
        self.formLayout_2.setWidget(10, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.groupBox_4)
        self.opt_emblem_size = QtWidgets.QSpinBox(parent=self.tab_5)
        self.opt_emblem_size.setMinimum(16)
        self.opt_emblem_size.setMaximum(1000)
        self.opt_emblem_size.setObjectName("opt_emblem_size")
        self.formLayout_2.setWidget(11, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_emblem_size)
        self.label_21 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_21.setObjectName("label_21")
        self.formLayout_2.setWidget(11, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_21)
        self.opt_cover_grid_show_title = QtWidgets.QCheckBox(parent=self.tab_5)
        self.opt_cover_grid_show_title.setObjectName("opt_cover_grid_show_title")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_cover_grid_show_title)
        self.label_22 = QtWidgets.QLabel(parent=self.tab_5)
        self.label_22.setObjectName("label_22")
        self.formLayout_2.setWidget(12, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_22)
        self.opt_emblem_position = QtWidgets.QComboBox(parent=self.tab_5)
        self.opt_emblem_position.setObjectName("opt_emblem_position")
        self.formLayout_2.setWidget(12, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_emblem_position)
        self.cover_grid_tab.addTab(self.tab_5, "")
        self.emblems_tab = QtWidgets.QWidget()
        self.emblems_tab.setObjectName("emblems_tab")
        self.cover_grid_tab.addTab(self.emblems_tab, "")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab_6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_3 = QtWidgets.QGroupBox(parent=self.tab_6)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout.setObjectName("gridLayout")
        self.label_13 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_13.setWordWrap(True)
        self.label_13.setObjectName("label_13")
        self.gridLayout.addWidget(self.label_13, 0, 0, 1, 5)
        self.opt_cover_grid_disk_cache_size = QtWidgets.QSpinBox(parent=self.groupBox_3)
        self.opt_cover_grid_disk_cache_size.setMinimum(100)
        self.opt_cover_grid_disk_cache_size.setSingleStep(100)
        self.opt_cover_grid_disk_cache_size.setObjectName("opt_cover_grid_disk_cache_size")
        self.gridLayout.addWidget(self.opt_cover_grid_disk_cache_size, 2, 2, 1, 1)
        self.cover_grid_empty_cache = QtWidgets.QPushButton(parent=self.groupBox_3)
        self.cover_grid_empty_cache.setObjectName("cover_grid_empty_cache")
        self.gridLayout.addWidget(self.cover_grid_empty_cache, 4, 0, 1, 1)
        self.cover_grid_open_cache = QtWidgets.QPushButton(parent=self.groupBox_3)
        self.cover_grid_open_cache.setObjectName("cover_grid_open_cache")
        self.gridLayout.addWidget(self.cover_grid_open_cache, 4, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(310, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem3, 4, 2, 1, 3)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem4, 5, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout.addItem(spacerItem5, 1, 3, 1, 1)
        self.label_15 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_15.setObjectName("label_15")
        self.gridLayout.addWidget(self.label_15, 1, 0, 1, 2)
        self.label_18 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_18.setObjectName("label_18")
        self.gridLayout.addWidget(self.label_18, 2, 0, 1, 2)
        self.opt_cover_grid_cache_size_multiple = QtWidgets.QSpinBox(parent=self.groupBox_3)
        self.opt_cover_grid_cache_size_multiple.setMinimum(2)
        self.opt_cover_grid_cache_size_multiple.setMaximum(100)
        self.opt_cover_grid_cache_size_multiple.setObjectName("opt_cover_grid_cache_size_multiple")
        self.gridLayout.addWidget(self.opt_cover_grid_cache_size_multiple, 1, 2, 1, 1)
        self.cover_grid_current_disk_cache = QtWidgets.QLabel(parent=self.groupBox_3)
        self.cover_grid_current_disk_cache.setText("")
        self.cover_grid_current_disk_cache.setObjectName("cover_grid_current_disk_cache")
        self.gridLayout.addWidget(self.cover_grid_current_disk_cache, 3, 2, 1, 1)
        self.verticalLayout_2.addWidget(self.groupBox_3)
        self.cover_grid_tab.addTab(self.tab_6, "")
        icon3 = QtGui.QIcon.ic("grid.png")
        self.tabWidget.addTab(self.cover_grid_tab, icon3, "")
        self.book_details_tab = QtWidgets.QWidget()
        self.book_details_tab.setObjectName("book_details_tab")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.book_details_tab)
        self.gridLayout_6.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.groupBox_5 = QtWidgets.QGroupBox(parent=self.book_details_tab)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.opt_book_details_css = QtWidgets.QPlainTextEdit(parent=self.groupBox_5)
        self.opt_book_details_css.setObjectName("opt_book_details_css")
        self.verticalLayout_3.addWidget(self.opt_book_details_css)
        self.gridLayout_6.addWidget(self.groupBox_5, 6, 0, 1, 1)
        self.id_links_button = QtWidgets.QPushButton(parent=self.book_details_tab)
        self.id_links_button.setObjectName("id_links_button")
        self.gridLayout_6.addWidget(self.id_links_button, 5, 0, 1, 3)
        self.default_author_link_container = QtWidgets.QWidget(parent=self.book_details_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.default_author_link_container.sizePolicy().hasHeightForWidth())
        self.default_author_link_container.setSizePolicy(sizePolicy)
        self.default_author_link_container.setObjectName("default_author_link_container")
        self.gridLayout_6.addWidget(self.default_author_link_container, 3, 0, 1, 3)
        self.groupBox3 = QtWidgets.QGroupBox(parent=self.book_details_tab)
        self.groupBox3.setObjectName("groupBox3")
        self.vblayout_3 = QtWidgets.QVBoxLayout(self.groupBox3)
        self.vblayout_3.setObjectName("vblayout_3")
        self.bd_vertical_cats = QtWidgets.QListView(parent=self.groupBox3)
        self.bd_vertical_cats.setAlternatingRowColors(True)
        self.bd_vertical_cats.setObjectName("bd_vertical_cats")
        self.vblayout_3.addWidget(self.bd_vertical_cats)
        self.gridLayout_6.addWidget(self.groupBox3, 6, 2, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(parent=self.book_details_tab)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.df_down_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon4 = QtGui.QIcon.ic("arrow-down.png")
        self.df_down_button.setIcon(icon4)
        self.df_down_button.setObjectName("df_down_button")
        self.gridLayout_3.addWidget(self.df_down_button, 3, 1, 1, 1)
        self.df_up_button = QtWidgets.QToolButton(parent=self.groupBox)
        icon5 = QtGui.QIcon.ic("arrow-up.png")
        self.df_up_button.setIcon(icon5)
        self.df_up_button.setObjectName("df_up_button")
        self.gridLayout_3.addWidget(self.df_up_button, 1, 1, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem6, 2, 1, 1, 1)
        self.field_display_order = ListViewWithMoveByKeyPress(parent=self.groupBox)
        self.field_display_order.setAlternatingRowColors(True)
        self.field_display_order.setObjectName("field_display_order")
        self.gridLayout_3.addWidget(self.field_display_order, 1, 0, 3, 1)
        self.label_3 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_3.setWordWrap(True)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 4, 0, 1, 2)
        self.gridLayout_6.addWidget(self.groupBox, 6, 1, 1, 1)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setObjectName("hboxlayout")
        self.opt_bd_show_cover = QtWidgets.QCheckBox(parent=self.book_details_tab)
        self.opt_bd_show_cover.setObjectName("opt_bd_show_cover")
        self.hboxlayout.addWidget(self.opt_bd_show_cover)
        self.opt_bd_overlay_cover_size = QtWidgets.QCheckBox(parent=self.book_details_tab)
        self.opt_bd_overlay_cover_size.setObjectName("opt_bd_overlay_cover_size")
        self.hboxlayout.addWidget(self.opt_bd_overlay_cover_size)
        self.opt_use_roman_numerals_for_series_number = QtWidgets.QCheckBox(parent=self.book_details_tab)
        self.opt_use_roman_numerals_for_series_number.setChecked(True)
        self.opt_use_roman_numerals_for_series_number.setObjectName("opt_use_roman_numerals_for_series_number")
        self.hboxlayout.addWidget(self.opt_use_roman_numerals_for_series_number)
        spacerItem7 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.hboxlayout.addItem(spacerItem7)
        self.gridLayout_6.addLayout(self.hboxlayout, 0, 0, 1, 3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_25 = QtWidgets.QLabel(parent=self.book_details_tab)
        self.label_25.setObjectName("label_25")
        self.horizontalLayout_2.addWidget(self.label_25)
        self.opt_book_details_comments_heading_pos = QtWidgets.QComboBox(parent=self.book_details_tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.opt_book_details_comments_heading_pos.sizePolicy().hasHeightForWidth())
        self.opt_book_details_comments_heading_pos.setSizePolicy(sizePolicy)
        self.opt_book_details_comments_heading_pos.setObjectName("opt_book_details_comments_heading_pos")
        self.horizontalLayout_2.addWidget(self.opt_book_details_comments_heading_pos)
        self.gridLayout_6.addLayout(self.horizontalLayout_2, 1, 0, 1, 2)
        icon6 = QtGui.QIcon.ic("book.png")
        self.tabWidget.addTab(self.book_details_tab, icon6, "")
        self.edit_metadata_tab = QtWidgets.QWidget()
        self.edit_metadata_tab.setObjectName("edit_metadata_tab")
        self.vbox_layout_61 = QtWidgets.QVBoxLayout(self.edit_metadata_tab)
        self.vbox_layout_61.setContentsMargins(0, 0, 0, 0)
        self.vbox_layout_61.setObjectName("vbox_layout_61")
        self.formlayout = QtWidgets.QFormLayout()
        self.formlayout.setObjectName("formlayout")
        self.edit_metadata_single_label = QtWidgets.QLabel(parent=self.edit_metadata_tab)
        self.edit_metadata_single_label.setObjectName("edit_metadata_single_label")
        self.formlayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.edit_metadata_single_label)
        self.opt_edit_metadata_single_layout = QtWidgets.QComboBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_single_layout.setObjectName("opt_edit_metadata_single_layout")
        self.formlayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_edit_metadata_single_layout)
        self.vbox_layout_61.addLayout(self.formlayout)
        self.groupBox1 = QtWidgets.QGroupBox(parent=self.edit_metadata_tab)
        self.groupBox1.setObjectName("groupBox1")
        self.gridLayout_31 = QtWidgets.QGridLayout(self.groupBox1)
        self.gridLayout_31.setObjectName("gridLayout_31")
        self.em_down_button = QtWidgets.QToolButton(parent=self.groupBox1)
        self.em_down_button.setIcon(icon4)
        self.em_down_button.setObjectName("em_down_button")
        self.gridLayout_31.addWidget(self.em_down_button, 3, 1, 1, 1)
        self.em_up_button = QtWidgets.QToolButton(parent=self.groupBox1)
        self.em_up_button.setIcon(icon5)
        self.em_up_button.setObjectName("em_up_button")
        self.gridLayout_31.addWidget(self.em_up_button, 1, 1, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_31.addItem(spacerItem8, 2, 1, 1, 1)
        self.em_display_order = ListViewWithMoveByKeyPress(parent=self.groupBox1)
        self.em_display_order.setAlternatingRowColors(True)
        self.em_display_order.setObjectName("em_display_order")
        self.gridLayout_31.addWidget(self.em_display_order, 1, 0, 3, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.em_reset_layout_button = QtWidgets.QPushButton(parent=self.groupBox1)
        self.em_reset_layout_button.setObjectName("em_reset_layout_button")
        self.horizontalLayout_4.addWidget(self.em_reset_layout_button)
        self.em_import_layout_button = QtWidgets.QPushButton(parent=self.groupBox1)
        self.em_import_layout_button.setObjectName("em_import_layout_button")
        self.horizontalLayout_4.addWidget(self.em_import_layout_button)
        self.em_export_layout_button = QtWidgets.QPushButton(parent=self.groupBox1)
        self.em_export_layout_button.setObjectName("em_export_layout_button")
        self.horizontalLayout_4.addWidget(self.em_export_layout_button)
        spacerItem9 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem9)
        self.gridLayout_31.addLayout(self.horizontalLayout_4, 5, 0, 1, 1)
        self.vbox_layout_61.addWidget(self.groupBox1)
        self.hboxlayout1 = QtWidgets.QHBoxLayout()
        self.hboxlayout1.setObjectName("hboxlayout1")
        self.formlayout1 = QtWidgets.QFormLayout()
        self.formlayout1.setObjectName("formlayout1")
        self.opt_edit_metadata_ignore_display_order = QtWidgets.QCheckBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_ignore_display_order.setObjectName("opt_edit_metadata_ignore_display_order")
        self.formlayout1.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_ignore_display_order)
        self.opt_edit_metadata_elide_labels = QtWidgets.QCheckBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_elide_labels.setObjectName("opt_edit_metadata_elide_labels")
        self.formlayout1.setWidget(1, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_elide_labels)
        self.label_1023 = QtWidgets.QLabel(parent=self.edit_metadata_tab)
        self.label_1023.setText("")
        self.label_1023.setObjectName("label_1023")
        self.formlayout1.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_1023)
        self.hboxlayout2 = QtWidgets.QHBoxLayout()
        self.hboxlayout2.setObjectName("hboxlayout2")
        self.label_102 = QtWidgets.QLabel(parent=self.edit_metadata_tab)
        self.label_102.setObjectName("label_102")
        self.hboxlayout2.addWidget(self.label_102)
        self.opt_edit_metadata_elision_point = QtWidgets.QComboBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_elision_point.setObjectName("opt_edit_metadata_elision_point")
        self.hboxlayout2.addWidget(self.opt_edit_metadata_elision_point)
        self.formlayout1.setLayout(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.hboxlayout2)
        self.opt_edit_metadata_templates_only_F2_on_booklist = QtWidgets.QCheckBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_templates_only_F2_on_booklist.setObjectName("opt_edit_metadata_templates_only_F2_on_booklist")
        self.formlayout1.setWidget(3, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_templates_only_F2_on_booklist)
        self.hboxlayout1.addLayout(self.formlayout1)
        self.formlayout2 = QtWidgets.QFormLayout()
        self.formlayout2.setObjectName("formlayout2")
        self.opt_edit_metadata_single_use_2_cols_for_custom_fields = QtWidgets.QCheckBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_single_use_2_cols_for_custom_fields.setObjectName("opt_edit_metadata_single_use_2_cols_for_custom_fields")
        self.formlayout2.setWidget(0, QtWidgets.QFormLayout.ItemRole.SpanningRole, self.opt_edit_metadata_single_use_2_cols_for_custom_fields)
        self.label_1021 = QtWidgets.QLabel(parent=self.edit_metadata_tab)
        self.label_1021.setObjectName("label_1021")
        self.formlayout2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_1021)
        self.opt_edit_metadata_bulk_cc_label_length = QtWidgets.QSpinBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_bulk_cc_label_length.setObjectName("opt_edit_metadata_bulk_cc_label_length")
        self.formlayout2.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_edit_metadata_bulk_cc_label_length)
        self.label_1022 = QtWidgets.QLabel(parent=self.edit_metadata_tab)
        self.label_1022.setObjectName("label_1022")
        self.formlayout2.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_1022)
        self.opt_edit_metadata_single_cc_label_length = QtWidgets.QSpinBox(parent=self.edit_metadata_tab)
        self.opt_edit_metadata_single_cc_label_length.setObjectName("opt_edit_metadata_single_cc_label_length")
        self.formlayout2.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_edit_metadata_single_cc_label_length)
        self.hboxlayout1.addLayout(self.formlayout2)
        self.vbox_layout_61.addLayout(self.hboxlayout1)
        icon7 = QtGui.QIcon.ic("edit_input.png")
        self.tabWidget.addTab(self.edit_metadata_tab, icon7, "")
        self.tag_browser_tab = QtWidgets.QTabWidget()
        self.tag_browser_tab.setObjectName("tag_browser_tab")
        self.tb_tab1 = QtWidgets.QWidget()
        self.tb_tab1.setObjectName("tb_tab1")
        self.verticalLayout_42 = QtWidgets.QGridLayout(self.tb_tab1)
        self.verticalLayout_42.setObjectName("verticalLayout_42")
        self.label = QtWidgets.QLabel(parent=self.tb_tab1)
        self.label.setObjectName("label")
        self.verticalLayout_42.addWidget(self.label, 0, 0, 1, 1)
        self.tb_display_order = ListViewWithMoveByKeyPress(parent=self.tb_tab1)
        self.tb_display_order.setMinimumSize(QtCore.QSize(0, 200))
        self.tb_display_order.setAlternatingRowColors(True)
        self.tb_display_order.setObjectName("tb_display_order")
        self.verticalLayout_42.addWidget(self.tb_display_order, 1, 0, 3, 1)
        self.tb_up_button = QtWidgets.QToolButton(parent=self.tb_tab1)
        self.tb_up_button.setIcon(icon5)
        self.tb_up_button.setObjectName("tb_up_button")
        self.verticalLayout_42.addWidget(self.tb_up_button, 1, 1, 1, 1)
        spacerItem10 = QtWidgets.QSpacerItem(50, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_42.addItem(spacerItem10, 2, 1, 1, 1)
        self.tb_down_button = QtWidgets.QToolButton(parent=self.tb_tab1)
        self.tb_down_button.setIcon(icon4)
        self.tb_down_button.setObjectName("tb_down_button")
        self.verticalLayout_42.addWidget(self.tb_down_button, 3, 1, 1, 1)
        self.horizontalLayout_41 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_41.setObjectName("horizontalLayout_41")
        self.tb_reset_layout_button = QtWidgets.QPushButton(parent=self.tb_tab1)
        self.tb_reset_layout_button.setObjectName("tb_reset_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_reset_layout_button)
        self.tb_import_layout_button = QtWidgets.QPushButton(parent=self.tb_tab1)
        self.tb_import_layout_button.setObjectName("tb_import_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_import_layout_button)
        self.tb_export_layout_button = QtWidgets.QPushButton(parent=self.tb_tab1)
        self.tb_export_layout_button.setObjectName("tb_export_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_export_layout_button)
        spacerItem11 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_41.addItem(spacerItem11)
        self.verticalLayout_42.addLayout(self.horizontalLayout_41, 4, 0, 1, 1)
        self.gridlayout_22 = QtWidgets.QGridLayout()
        self.gridlayout_22.setObjectName("gridlayout_22")
        self.opt_show_avg_rating = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_show_avg_rating.setChecked(True)
        self.opt_show_avg_rating.setObjectName("opt_show_avg_rating")
        self.gridlayout_22.addWidget(self.opt_show_avg_rating, 0, 0, 1, 1)
        self.opt_show_links_in_tag_browser = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_show_links_in_tag_browser.setChecked(True)
        self.opt_show_links_in_tag_browser.setObjectName("opt_show_links_in_tag_browser")
        self.gridlayout_22.addWidget(self.opt_show_links_in_tag_browser, 0, 1, 1, 1)
        self.opt_tag_browser_show_tooltips = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_tag_browser_show_tooltips.setObjectName("opt_tag_browser_show_tooltips")
        self.gridlayout_22.addWidget(self.opt_tag_browser_show_tooltips, 0, 2, 1, 1)
        self.opt_tag_browser_show_counts = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_tag_browser_show_counts.setObjectName("opt_tag_browser_show_counts")
        self.gridlayout_22.addWidget(self.opt_tag_browser_show_counts, 1, 0, 1, 1)
        self.opt_show_notes_in_tag_browser = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_show_notes_in_tag_browser.setChecked(True)
        self.opt_show_notes_in_tag_browser.setObjectName("opt_show_notes_in_tag_browser")
        self.gridlayout_22.addWidget(self.opt_show_notes_in_tag_browser, 1, 1, 1, 1)
        self.opt_tag_browser_old_look = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_tag_browser_old_look.setObjectName("opt_tag_browser_old_look")
        self.gridlayout_22.addWidget(self.opt_tag_browser_old_look, 1, 2, 1, 1)
        self.opt_tag_browser_hide_empty_categories = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_tag_browser_hide_empty_categories.setObjectName("opt_tag_browser_hide_empty_categories")
        self.gridlayout_22.addWidget(self.opt_tag_browser_hide_empty_categories, 2, 0, 1, 1)
        self.opt_icons_on_right_in_tag_browser = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_icons_on_right_in_tag_browser.setChecked(True)
        self.opt_icons_on_right_in_tag_browser.setObjectName("opt_icons_on_right_in_tag_browser")
        self.gridlayout_22.addWidget(self.opt_icons_on_right_in_tag_browser, 2, 1, 1, 1)
        self.opt_tag_browser_always_autocollapse = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_tag_browser_always_autocollapse.setObjectName("opt_tag_browser_always_autocollapse")
        self.gridlayout_22.addWidget(self.opt_tag_browser_always_autocollapse, 2, 2, 1, 1)
        self.verticalLayout_42.addLayout(self.gridlayout_22, 5, 0, 1, 3)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.opt_tag_browser_allow_keyboard_focus = QtWidgets.QCheckBox(parent=self.tb_tab1)
        self.opt_tag_browser_allow_keyboard_focus.setObjectName("opt_tag_browser_allow_keyboard_focus")
        self.verticalLayout.addWidget(self.opt_tag_browser_allow_keyboard_focus)
        self.tb_focus_label = QtWidgets.QLabel(parent=self.tb_tab1)
        self.tb_focus_label.setStyleSheet("margin-left: 1.5em")
        self.tb_focus_label.setWordWrap(True)
        self.tb_focus_label.setObjectName("tb_focus_label")
        self.verticalLayout.addWidget(self.tb_focus_label)
        self.verticalLayout_42.addLayout(self.verticalLayout, 8, 0, 1, 3)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label1 = QtWidgets.QLabel(parent=self.tb_tab1)
        self.label1.setObjectName("label1")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label1)
        self.opt_tag_browser_item_padding = QtWidgets.QDoubleSpinBox(parent=self.tb_tab1)
        self.opt_tag_browser_item_padding.setDecimals(1)
        self.opt_tag_browser_item_padding.setMinimum(-1.0)
        self.opt_tag_browser_item_padding.setMaximum(2.0)
        self.opt_tag_browser_item_padding.setSingleStep(0.1)
        self.opt_tag_browser_item_padding.setObjectName("opt_tag_browser_item_padding")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tag_browser_item_padding)
        self.verticalLayout_42.addLayout(self.formLayout, 10, 0, 1, 3)
        self.tag_browser_tab.addTab(self.tb_tab1, "")
        self.tb_tab2 = QtWidgets.QWidget()
        self.tb_tab2.setObjectName("tb_tab2")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tb_tab2)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.gridLayout_10 = QtWidgets.QGridLayout()
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.formLayout1 = QtWidgets.QFormLayout()
        self.formLayout1.setFieldGrowthPolicy(QtWidgets.QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
        self.formLayout1.setObjectName("formLayout1")
        self.label_9 = QtWidgets.QLabel(parent=self.tb_tab2)
        self.label_9.setObjectName("label_9")
        self.formLayout1.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.opt_tags_browser_partition_method = QtWidgets.QComboBox(parent=self.tb_tab2)
        self.opt_tags_browser_partition_method.setObjectName("opt_tags_browser_partition_method")
        self.formLayout1.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tags_browser_partition_method)
        self.label_10 = QtWidgets.QLabel(parent=self.tb_tab2)
        self.label_10.setObjectName("label_10")
        self.formLayout1.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.opt_tags_browser_collapse_fl_at = QtWidgets.QSpinBox(parent=self.tb_tab2)
        self.opt_tags_browser_collapse_fl_at.setMaximum(10000)
        self.opt_tags_browser_collapse_fl_at.setObjectName("opt_tags_browser_collapse_fl_at")
        self.formLayout1.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tags_browser_collapse_fl_at)
        self.label_101 = QtWidgets.QLabel(parent=self.tb_tab2)
        self.label_101.setObjectName("label_101")
        self.formLayout1.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_101)
        self.opt_tags_browser_collapse_at = QtWidgets.QSpinBox(parent=self.tb_tab2)
        self.opt_tags_browser_collapse_at.setMaximum(10000)
        self.opt_tags_browser_collapse_at.setObjectName("opt_tags_browser_collapse_at")
        self.formLayout1.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.opt_tags_browser_collapse_at)
        self.gridLayout_10.addLayout(self.formLayout1, 0, 0, 1, 1)
        self.label2 = QtWidgets.QLabel(parent=self.tb_tab2)
        self.label2.setObjectName("label2")
        self.gridLayout_10.addWidget(self.label2, 1, 0, 1, 1)
        self.tb_cats_to_partition = QtWidgets.QListView(parent=self.tb_tab2)
        self.tb_cats_to_partition.setMinimumSize(QtCore.QSize(0, 200))
        self.tb_cats_to_partition.setAlternatingRowColors(True)
        self.tb_cats_to_partition.setObjectName("tb_cats_to_partition")
        self.gridLayout_10.addWidget(self.tb_cats_to_partition, 2, 0, 1, 1)
        self.horizontalLayout_41 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_41.setObjectName("horizontalLayout_41")
        self.tb_partition_reset_button = QtWidgets.QPushButton(parent=self.tb_tab2)
        self.tb_partition_reset_button.setObjectName("tb_partition_reset_button")
        self.horizontalLayout_41.addWidget(self.tb_partition_reset_button)
        self.tb_partition_import_layout_button = QtWidgets.QPushButton(parent=self.tb_tab2)
        self.tb_partition_import_layout_button.setObjectName("tb_partition_import_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_partition_import_layout_button)
        self.tb_partition_export_layout_button = QtWidgets.QPushButton(parent=self.tb_tab2)
        self.tb_partition_export_layout_button.setObjectName("tb_partition_export_layout_button")
        self.horizontalLayout_41.addWidget(self.tb_partition_export_layout_button)
        spacerItem12 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_41.addItem(spacerItem12)
        self.gridLayout_10.addLayout(self.horizontalLayout_41, 3, 0, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout_10)
        self.tag_browser_tab.addTab(self.tb_tab2, "")
        self.tb_tab3 = QtWidgets.QWidget()
        self.tb_tab3.setObjectName("tb_tab3")
        self.gridlayout = QtWidgets.QGridLayout(self.tb_tab3)
        self.gridlayout.setObjectName("gridlayout")
        self.label3 = QtWidgets.QLabel(parent=self.tb_tab3)
        self.label3.setObjectName("label3")
        self.gridlayout.addWidget(self.label3, 0, 0, 1, 1)
        self.tb_hierarchical_cats = QtWidgets.QListView(parent=self.tb_tab3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.tb_hierarchical_cats.sizePolicy().hasHeightForWidth())
        self.tb_hierarchical_cats.setSizePolicy(sizePolicy)
        self.tb_hierarchical_cats.setMinimumSize(QtCore.QSize(0, 200))
        self.tb_hierarchical_cats.setAlternatingRowColors(True)
        self.tb_hierarchical_cats.setObjectName("tb_hierarchical_cats")
        self.gridlayout.addWidget(self.tb_hierarchical_cats, 1, 0, 3, 1)
        self.horizontalLayout_42 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_42.setObjectName("horizontalLayout_42")
        self.tb_hierarchy_reset_layout_button = QtWidgets.QPushButton(parent=self.tb_tab3)
        self.tb_hierarchy_reset_layout_button.setObjectName("tb_hierarchy_reset_layout_button")
        self.horizontalLayout_42.addWidget(self.tb_hierarchy_reset_layout_button)
        self.tb_hierarchy_import_layout_button = QtWidgets.QPushButton(parent=self.tb_tab3)
        self.tb_hierarchy_import_layout_button.setObjectName("tb_hierarchy_import_layout_button")
        self.horizontalLayout_42.addWidget(self.tb_hierarchy_import_layout_button)
        self.tb_hierarchy_export_layout_button = QtWidgets.QPushButton(parent=self.tb_tab3)
        self.tb_hierarchy_export_layout_button.setObjectName("tb_hierarchy_export_layout_button")
        self.horizontalLayout_42.addWidget(self.tb_hierarchy_export_layout_button)
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_42.addItem(spacerItem13)
        self.gridlayout.addLayout(self.horizontalLayout_42, 4, 0, 1, 1)
        spacerItem14 = QtWidgets.QSpacerItem(0, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridlayout.addItem(spacerItem14, 10, 0, 1, 1)
        self.label4 = QtWidgets.QLabel(parent=self.tb_tab3)
        self.label4.setObjectName("label4")
        self.gridlayout.addWidget(self.label4, 11, 0, 1, 1)
        self.tb_search_order = ListWidgetWithMoveByKeyPress(parent=self.tb_tab3)
        self.tb_search_order.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.tb_search_order.setObjectName("tb_search_order")
        self.gridlayout.addWidget(self.tb_search_order, 12, 0, 3, 1)
        self.tb_search_order_up_button = QtWidgets.QToolButton(parent=self.tb_tab3)
        self.tb_search_order_up_button.setIcon(icon5)
        self.tb_search_order_up_button.setObjectName("tb_search_order_up_button")
        self.gridlayout.addWidget(self.tb_search_order_up_button, 12, 1, 1, 1)
        spacerItem15 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridlayout.addItem(spacerItem15, 13, 1, 1, 1)
        self.tb_search_order_down_button = QtWidgets.QToolButton(parent=self.tb_tab3)
        self.tb_search_order_down_button.setIcon(icon4)
        self.tb_search_order_down_button.setObjectName("tb_search_order_down_button")
        self.gridlayout.addWidget(self.tb_search_order_down_button, 14, 1, 1, 1)
        self.horizontalLayout_43 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_43.setObjectName("horizontalLayout_43")
        self.tb_search_order_reset_button = QtWidgets.QPushButton(parent=self.tb_tab3)
        self.tb_search_order_reset_button.setObjectName("tb_search_order_reset_button")
        self.horizontalLayout_43.addWidget(self.tb_search_order_reset_button)
        spacerItem16 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_43.addItem(spacerItem16)
        self.gridlayout.addLayout(self.horizontalLayout_43, 15, 0, 1, 1)
        self.tag_browser_tab.addTab(self.tb_tab3, "")
        icon8 = QtGui.QIcon.ic("tags.png")
        self.tabWidget.addTab(self.tag_browser_tab, icon8, "")
        self.cover_browser_tab = QtWidgets.QWidget()
        self.cover_browser_tab.setObjectName("cover_browser_tab")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.cover_browser_tab)
        self.gridLayout_11.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.opt_cb_fullscreen = QtWidgets.QCheckBox(parent=self.cover_browser_tab)
        self.opt_cb_fullscreen.setObjectName("opt_cb_fullscreen")
        self.gridLayout_11.addWidget(self.opt_cb_fullscreen, 0, 1, 1, 2)
        self.opt_separate_cover_flow = QtWidgets.QCheckBox(parent=self.cover_browser_tab)
        self.opt_separate_cover_flow.setObjectName("opt_separate_cover_flow")
        self.gridLayout_11.addWidget(self.opt_separate_cover_flow, 0, 0, 1, 3)
        self.opt_cover_flow_queue_length = QtWidgets.QSpinBox(parent=self.cover_browser_tab)
        self.opt_cover_flow_queue_length.setObjectName("opt_cover_flow_queue_length")
        self.gridLayout_11.addWidget(self.opt_cover_flow_queue_length, 7, 1, 1, 2)
        self.label_61 = QtWidgets.QLabel(parent=self.cover_browser_tab)
        self.label_61.setObjectName("label_61")
        self.gridLayout_11.addWidget(self.label_61, 7, 0, 1, 1)
        self.cover_browser_title_template_button = QtWidgets.QPushButton(parent=self.cover_browser_tab)
        self.cover_browser_title_template_button.setObjectName("cover_browser_title_template_button")
        self.gridLayout_11.addWidget(self.cover_browser_title_template_button, 8, 2, 1, 1)
        spacerItem17 = QtWidgets.QSpacerItem(690, 20, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_11.addItem(spacerItem17, 11, 0, 1, 2)
        self.opt_cover_browser_title_template = QtWidgets.QLineEdit(parent=self.cover_browser_tab)
        self.opt_cover_browser_title_template.setObjectName("opt_cover_browser_title_template")
        self.gridLayout_11.addWidget(self.opt_cover_browser_title_template, 8, 1, 1, 1)
        self.opt_cover_browser_reflections = QtWidgets.QCheckBox(parent=self.cover_browser_tab)
        self.opt_cover_browser_reflections.setObjectName("opt_cover_browser_reflections")
        self.gridLayout_11.addWidget(self.opt_cover_browser_reflections, 4, 0, 1, 1)
        self.opt_cb_preserve_aspect_ratio = QtWidgets.QCheckBox(parent=self.cover_browser_tab)
        self.opt_cb_preserve_aspect_ratio.setObjectName("opt_cb_preserve_aspect_ratio")
        self.gridLayout_11.addWidget(self.opt_cb_preserve_aspect_ratio, 4, 1, 1, 2)
        self.label_987 = QtWidgets.QLabel(parent=self.cover_browser_tab)
        self.label_987.setObjectName("label_987")
        self.gridLayout_11.addWidget(self.label_987, 6, 0, 1, 1)
        self.opt_cover_browser_narrow_view_position = QtWidgets.QComboBox(parent=self.cover_browser_tab)
        self.opt_cover_browser_narrow_view_position.setObjectName("opt_cover_browser_narrow_view_position")
        self.gridLayout_11.addWidget(self.opt_cover_browser_narrow_view_position, 6, 1, 1, 2)
        self.opt_cover_browser_subtitle_field = QtWidgets.QComboBox(parent=self.cover_browser_tab)
        self.opt_cover_browser_subtitle_field.setObjectName("opt_cover_browser_subtitle_field")
        self.gridLayout_11.addWidget(self.opt_cover_browser_subtitle_field, 9, 1, 1, 2)
        self.label_23 = QtWidgets.QLabel(parent=self.cover_browser_tab)
        self.label_23.setObjectName("label_23")
        self.gridLayout_11.addWidget(self.label_23, 8, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(parent=self.cover_browser_tab)
        self.label_24.setObjectName("label_24")
        self.gridLayout_11.addWidget(self.label_24, 9, 0, 1, 1)
        self.fs_help_msg = QtWidgets.QLabel(parent=self.cover_browser_tab)
        self.fs_help_msg.setStyleSheet("margin-left: 1.5em")
        self.fs_help_msg.setWordWrap(True)
        self.fs_help_msg.setObjectName("fs_help_msg")
        self.gridLayout_11.addWidget(self.fs_help_msg, 1, 1, 1, 2)
        self.label_26 = QtWidgets.QLabel(parent=self.cover_browser_tab)
        self.label_26.setObjectName("label_26")
        self.gridLayout_11.addWidget(self.label_26, 10, 0, 1, 1)
        self.opt_books_autoscroll_time = QtWidgets.QDoubleSpinBox(parent=self.cover_browser_tab)
        self.opt_books_autoscroll_time.setDecimals(1)
        self.opt_books_autoscroll_time.setMinimum(1.0)
        self.opt_books_autoscroll_time.setMaximum(100000.0)
        self.opt_books_autoscroll_time.setObjectName("opt_books_autoscroll_time")
        self.gridLayout_11.addWidget(self.opt_books_autoscroll_time, 10, 1, 1, 1)
        self.opt_cb_double_click_to_activate = QtWidgets.QCheckBox(parent=self.cover_browser_tab)
        self.opt_cb_double_click_to_activate.setObjectName("opt_cb_double_click_to_activate")
        self.gridLayout_11.addWidget(self.opt_cb_double_click_to_activate, 5, 0, 1, 2)
        icon9 = QtGui.QIcon.ic("cover_flow.png")
        self.tabWidget.addTab(self.cover_browser_tab, icon9, "")
        self.quickview_tab = QtWidgets.QWidget()
        self.quickview_tab.setObjectName("quickview_tab")
        self.gridLayout_122 = QtWidgets.QGridLayout(self.quickview_tab)
        self.gridLayout_122.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_122.setObjectName("gridLayout_122")
        self.gridLayout_1221 = QtWidgets.QGridLayout()
        self.gridLayout_1221.setObjectName("gridLayout_1221")
        self.opt_qv_respects_vls = QtWidgets.QCheckBox(parent=self.quickview_tab)
        self.opt_qv_respects_vls.setObjectName("opt_qv_respects_vls")
        self.gridLayout_1221.addWidget(self.opt_qv_respects_vls, 2, 0, 1, 1)
        self.opt_qv_follows_column = QtWidgets.QCheckBox(parent=self.quickview_tab)
        self.opt_qv_follows_column.setObjectName("opt_qv_follows_column")
        self.gridLayout_1221.addWidget(self.opt_qv_follows_column, 2, 1, 1, 1)
        self.opt_qv_retkey_changes_column = QtWidgets.QCheckBox(parent=self.quickview_tab)
        self.opt_qv_retkey_changes_column.setObjectName("opt_qv_retkey_changes_column")
        self.gridLayout_1221.addWidget(self.opt_qv_retkey_changes_column, 1, 0, 1, 1)
        self.opt_qv_dclick_changes_column = QtWidgets.QCheckBox(parent=self.quickview_tab)
        self.opt_qv_dclick_changes_column.setObjectName("opt_qv_dclick_changes_column")
        self.gridLayout_1221.addWidget(self.opt_qv_dclick_changes_column, 1, 1, 1, 1)
        self.gridLayout_122.addLayout(self.gridLayout_1221, 1, 0, 1, 2)
        self.groupBox2 = QtWidgets.QGroupBox(parent=self.quickview_tab)
        self.groupBox2.setObjectName("groupBox2")
        self.gridLayout_32 = QtWidgets.QGridLayout(self.groupBox2)
        self.gridLayout_32.setObjectName("gridLayout_32")
        self.qv_up_button = QtWidgets.QToolButton(parent=self.groupBox2)
        self.qv_up_button.setIcon(icon5)
        self.qv_up_button.setObjectName("qv_up_button")
        self.gridLayout_32.addWidget(self.qv_up_button, 0, 1, 1, 1)
        self.qv_down_button = QtWidgets.QToolButton(parent=self.groupBox2)
        self.qv_down_button.setIcon(icon4)
        self.qv_down_button.setObjectName("qv_down_button")
        self.gridLayout_32.addWidget(self.qv_down_button, 2, 1, 1, 1)
        self.qv_display_order = ListViewWithMoveByKeyPress(parent=self.groupBox2)
        self.qv_display_order.setAlternatingRowColors(True)
        self.qv_display_order.setObjectName("qv_display_order")
        self.gridLayout_32.addWidget(self.qv_display_order, 0, 0, 3, 1)
        spacerItem18 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_32.addItem(spacerItem18, 1, 1, 1, 1)
        self.gridLayout_122.addWidget(self.groupBox2, 3, 0, 2, 1)
        icon10 = QtGui.QIcon.ic("quickview.png")
        self.tabWidget.addTab(self.quickview_tab, icon10, "")
        self.gridLayout_2.addWidget(self.tabWidget, 0, 1, 1, 1)
        self.sections_view = QtWidgets.QListWidget(parent=Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sections_view.sizePolicy().hasHeightForWidth())
        self.sections_view.setSizePolicy(sizePolicy)
        self.sections_view.setObjectName("sections_view")
        self.gridLayout_2.addWidget(self.sections_view, 0, 0, 1, 1)
        self.label_widget_style.setBuddy(self.opt_ui_style)
        self.opt_color_palette_label.setBuddy(self.button_adjust_colors)
        self.label_17.setBuddy(self.opt_gui_layout)
        self.label_7.setBuddy(self.opt_language)
        self.label_6.setBuddy(self.opt_cover_corner_radius)
        self.label_2.setBuddy(self.font_display)
        self.label_5.setBuddy(self.opt_toolbar_icon_size)
        self.label_8.setBuddy(self.opt_toolbar_text)
        self.label_4.setBuddy(self.opt_book_list_extra_row_spacing)
        self.label_14.setBuddy(self.opt_field_under_covers_in_grid)
        self.label_16.setBuddy(self.opt_cover_grid_spacing)
        self.label_12.setBuddy(self.opt_cover_grid_height)
        self.label_11.setBuddy(self.opt_cover_grid_width)
        self.label_21.setBuddy(self.opt_emblem_size)
        self.label_22.setBuddy(self.opt_emblem_position)
        self.label_15.setBuddy(self.opt_cover_grid_cache_size_multiple)
        self.label_18.setBuddy(self.opt_cover_grid_disk_cache_size)
        self.label_25.setBuddy(self.opt_book_details_comments_heading_pos)
        self.edit_metadata_single_label.setBuddy(self.opt_edit_metadata_single_layout)
        self.label_102.setBuddy(self.opt_edit_metadata_elision_point)
        self.label_1021.setBuddy(self.opt_edit_metadata_bulk_cc_label_length)
        self.label_1022.setBuddy(self.opt_edit_metadata_single_cc_label_length)
        self.label.setBuddy(self.tb_display_order)
        self.label1.setBuddy(self.opt_tag_browser_item_padding)
        self.label_9.setBuddy(self.opt_tags_browser_partition_method)
        self.label_10.setBuddy(self.opt_tags_browser_collapse_fl_at)
        self.label_101.setBuddy(self.opt_tags_browser_collapse_at)
        self.label2.setBuddy(self.tb_cats_to_partition)
        self.label3.setBuddy(self.tb_hierarchical_cats)
        self.label4.setBuddy(self.tb_search_order)
        self.label_61.setBuddy(self.opt_cover_flow_queue_length)
        self.label_987.setBuddy(self.opt_cover_browser_narrow_view_position)
        self.label_23.setBuddy(self.opt_cover_browser_title_template)
        self.label_24.setBuddy(self.opt_cover_browser_subtitle_field)
        self.label_26.setBuddy(self.opt_books_autoscroll_time)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(0)
        self.cover_grid_tab.setCurrentIndex(0)
        self.opt_cover_grid_show_title.toggled['bool'].connect(self.opt_field_under_covers_in_grid.setEnabled) # type: ignore
        self.opt_tag_browser_allow_keyboard_focus.toggled['bool'].connect(self.tb_focus_label.setVisible) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):

        Form.setWindowTitle(_("Form"))
        self.label_widget_style.setText(_("User interface style (&needs restart):"))
        self.icon_theme_button.setText(_("Change &icon theme"))
        self.opt_color_palette_label.setText(_("User interface &colors:"))
        self.button_adjust_colors.setText(_("Adjust &colors"))
        self.label_17.setText(_("&User interface layout:"))
        self.label_7.setText(_("Choose &language (needs restart):"))
        self.label_6.setText(_("&Round the corners of covers:"))
        self.opt_cover_corner_radius.setToolTip(_("Round the corners of covers in many places they are displayed such as the Cover grid, Book details panel, etc. Adjust the amount of rounding with this setting. A value of between 5 and 10 looks good with most covers sizes. Zero disables rounding and is the default."))
        self.opt_cover_corner_radius.setSpecialValueText(_("no rounding"))
        self.opt_cover_corner_radius_unit.setToolTip(_("The unit for the cover corner rounding. Either pixel values or as a percentage of the cover size."))
        self.opt_systray_icon.setText(_("Enable s&ystem tray icon (needs restart)"))
        self.opt_disable_animations.setToolTip(_("Disable all animations. Useful if you have a slow/old computer."))
        self.opt_disable_animations.setText(_("Disable &animations"))
        self.opt_disable_tray_notification.setToolTip(_("Disable popup notifications when calibre completes jobs such a conversion, sending to device etc. The notifications are sent via the operating system notification facility, if available. Note that on Windows, you have to enable the system tray icon for notifications to work."))
        self.opt_disable_tray_notification.setText(_("Disable n&otifications on job completion"))
        self.opt_show_splash_screen.setText(_("Show the &splash screen at startup"))
        self.opt_book_list_tooltips.setText(_("Show &tooltips in the book list"))
        self.opt_show_layout_buttons.setText(_("Show &layout buttons in the status bar"))
        self.label_2.setText(_("&Interface font:"))
        self.change_font_button.setText(_("Change &font (needs restart)"))
        self.groupBox_2.setTitle(_("Toolbar"))
        self.label_5.setText(_("Icon si&ze:"))
        self.label_8.setText(_("Show &text under icons:"))
        self.opt_wrap_toolbar_text.setText(_("Use t&wo lines for the text under the icons (needs restart)"))
        self.label_4.setText(_("E&xtra spacing to add between rows in the book list (can be negative):"))
        self.opt_book_list_extra_row_spacing.setSuffix(_(" px"))
        self.opt_row_numbers_in_book_list.setText(_("Show &row numbers in the book list"))
        self.opt_booklist_grid.setText(_("Draw a &grid in the book list"))
        self.opt_dnd_merge.setText(_("Allow using &drag and drop to merge books"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.main_interface_tab), _("&Main interface"))
        self.label_19.setText(_("Control the Cover grid view. You can enable this view by clicking the \"Layout\" button in the bottom right corner of the main calibre window."))
        self.label_14.setText(_("Field to show &under the covers:"))
        self.label_16.setText(_("Spac&ing between covers:"))
        self.opt_cover_grid_spacing.setToolTip(_("The spacing between covers. A value of zero means calculate automatically based on cover size."))
        self.opt_cover_grid_spacing.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_spacing.setSuffix(_(" cm"))
        self.cg_background_box.setTitle(_("Background for the Cover grid"))
        self.groupBox_4.setTitle(_("Cover size"))
        self.cover_grid_larger_cover.setToolTip(_("Make the covers larger, maintaining current aspect ratio."))
        self.cover_grid_larger_cover.setText(_("&Larger covers"))
        self.cover_grid_smaller_cover.setToolTip(_("Make the covers smaller, maintaining current aspect ratio."))
        self.cover_grid_smaller_cover.setText(_("&Smaller covers"))
        self.label_12.setText(_("Cover &height: "))
        self.label_11.setText(_("Cover &width: "))
        self.opt_cover_grid_height.setToolTip(_("The height of displayed covers.\n"
"A value of zero means calculate automatically."))
        self.opt_cover_grid_height.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_height.setSuffix(_(" cm"))
        self.cover_grid_reset_size.setToolTip(_("Reset size to automatic"))
        self.cover_grid_reset_size.setText(_("&Reset size"))
        self.opt_cover_grid_width.setToolTip(_("The width of displayed covers.\n"
"A value of zero means calculate automatically."))
        self.opt_cover_grid_width.setSpecialValueText(_("Automatic"))
        self.opt_cover_grid_width.setSuffix(_(" cm"))
        self.cover_size_label.setText(_("By default, calibre chooses a cover size based on your computer\'s screen size. You can change the cover size here:"))
        self.opt_emblem_size.setSuffix(_(" px"))
        self.label_21.setText(_("Size of the emblems (if any) shown &next to the covers: "))
        self.opt_cover_grid_show_title.setText(_("Show a &field (such as title) under the covers"))
        self.label_22.setText(_("&Location of the emblems shown next to the covers:"))
        self.cover_grid_tab.setTabText(self.cover_grid_tab.indexOf(self.tab_5), _("&Layout"))
        self.cover_grid_tab.setTabText(self.cover_grid_tab.indexOf(self.emblems_tab), _("&Emblems"))
        self.groupBox_3.setTitle(_("Caching of covers for improved performance"))
        self.label_13.setText(_("There are two kinds of caches that calibre uses to improve performance when rendering covers in the grid view. A disk cache that is kept on your hard disk and stores the cover thumbnails and an in memory cache used to ensure flicker free rendering of covers. For best results, keep the memory cache small and the disk cache large, unless you have a lot of extra RAM in your computer and don\'t mind it being used by the memory cache."))
        self.opt_cover_grid_disk_cache_size.setSpecialValueText(_("Disable"))
        self.opt_cover_grid_disk_cache_size.setSuffix(_(" MB"))
        self.cover_grid_empty_cache.setText(_("&Empty disk cache"))
        self.cover_grid_open_cache.setText(_("&Open cache folder"))
        self.label_15.setText(_("Number of screenfulls of covers to cache in &memory (keep this small):"))
        self.label_18.setText(_("Maximum amount of &disk space to use for caching thumbnails: "))
        self.opt_cover_grid_cache_size_multiple.setToolTip(_("The maximum number of screenfulls of thumbnails to keep in memory. Increasing this will make rendering faster, at the cost of more memory usage. Note that regardless of this setting, a minimum of one hundred thumbnails are always kept in memory, to ensure flicker free rendering."))
        self.cover_grid_tab.setTabText(self.cover_grid_tab.indexOf(self.tab_6), _("&Performance"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cover_grid_tab), _("Cover &grid"))
        self.groupBox_5.setTitle(_("Text styling"))
        self.id_links_button.setText(_("Create rules to convert &identifiers into links"))
        self.groupBox3.setToolTip(_("<p>Check the box if you want the category\'s values displayed on separate lines instead of separated by commas</p>"))
        self.groupBox3.setTitle(_("Categories on separate lines"))
        self.groupBox.setTitle(_("Select displayed metadata"))
        self.df_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.df_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.label_3.setText(_("<p>Note: <b>comments</b>-like columns will always\n"
"                     be displayed at the end unless their \"Heading position\" is\n"
"                     \"Show heading to the side\".</p>"))
        self.opt_bd_show_cover.setText(_("Show &cover"))
        self.opt_bd_overlay_cover_size.setToolTip(_("Show the size of the book\'s cover in pixels"))
        self.opt_bd_overlay_cover_size.setText(_("Show cover &size"))
        self.opt_use_roman_numerals_for_series_number.setText(_("Use &Roman numerals for series"))
        self.label_25.setText(_("Show comments &heading:"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.book_details_tab), _("&Book details"))
        self.edit_metadata_single_label.setText(_("Edit metadata (single) &layout:"))
        self.opt_edit_metadata_single_layout.setToolTip(_("Choose a different layout for the Edit metadata dialog. The compact metadata layout favors editing custom metadata over changing covers and formats."))
        self.groupBox1.setTitle(_("Select the custom columns (for this library) to display in the edit metadata dialogs and their order:"))
        self.em_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.em_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.em_reset_layout_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.em_reset_layout_button.setText(_("Reset list"))
        self.em_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same column order for each one. Columns\n"
"in the imported list that aren\'t in the current library are ignored. Columns in\n"
"the library that are not in the imported list are put at the end and marked\n"
"as displayable.</p>"))
        self.em_import_layout_button.setText(_("Import list"))
        self.em_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same column order for each one.</p>"))
        self.em_export_layout_button.setText(_("Export list"))
        self.opt_edit_metadata_ignore_display_order.setToolTip(_("<p>Check this box to make the edit metadata dialogs ignore the\n"
"above specifications, showing all the columns in the default order. This is\n"
"useful for temporarily seeing all your columns in the dialogs without losing\n"
"the display and order specifications.</p>"))
        self.opt_edit_metadata_ignore_display_order.setText(_("Show &all columns in default order when editing metadata"))
        self.opt_edit_metadata_elide_labels.setToolTip(_("If checked then labels wider than the label width\n"
"will be elided, otherwise they will be word wrapped."))
        self.opt_edit_metadata_elide_labels.setText(_("&Elide labels when editing custom columns"))
        self.label_102.setText(_("Elision point:"))
        self.opt_edit_metadata_elision_point.setToolTip(_("Choose where in the label to put the..."))
        self.opt_edit_metadata_templates_only_F2_on_booklist.setToolTip(_("<p>Check this box to allow only the F2 (Edit) key to\n"
"open the template editor in the book list for a \"Column built from other\n"
"columns\". Editing with mouse clicks and the Tab key will be disabled.</p>"))
        self.opt_edit_metadata_templates_only_F2_on_booklist.setText(_("Only the &F2 (Edit) key edits column templates in the book list"))
        self.opt_edit_metadata_single_use_2_cols_for_custom_fields.setText(_("Use &two columns for custom columns in the Default layout"))
        self.label_1021.setText(_("&Bulk edit custom column label length:"))
        self.opt_edit_metadata_bulk_cc_label_length.setToolTip(_("The maximum width of a custom column label for the bulk metadata edit dialog in average characters."))
        self.opt_edit_metadata_bulk_cc_label_length.setSuffix(_(" chars"))
        self.label_1022.setText(_("&Single edit custom column label length:"))
        self.opt_edit_metadata_single_cc_label_length.setToolTip(_("The maximum width of a custom column label for the single metadata edit dialog in average characters."))
        self.opt_edit_metadata_single_cc_label_length.setSuffix(_(" chars"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.edit_metadata_tab), _("Edit &metadata"))
        self.label.setToolTip(_("User categories and Saved searches cannot be moved"))
        self.label.setText(_("Select the categories to display in the Tag browser, and their &order"))
        self.tb_up_button.setToolTip(_("Move up. User categories and Saved searches cannot be moved. Keyboard shortcut: Ctrl-Up arrow"))
        self.tb_down_button.setToolTip(_("Move down. User categories and Saved searches cannot be moved. Keyboard shortcut: Ctrl-Down arrow"))
        self.tb_reset_layout_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_reset_layout_button.setText(_("Reset list"))
        self.tb_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same column order for each one. Columns\n"
"in the imported list that aren\'t in the current library are ignored. Columns in\n"
"the library that are not in the imported list are put at the end and marked\n"
"as displayable.</p>"))
        self.tb_import_layout_button.setText(_("Import list"))
        self.tb_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same column order for each one.</p>"))
        self.tb_export_layout_button.setText(_("Export list"))
        self.opt_show_avg_rating.setToolTip(_("Show the average rating per item indication in the Tag browser"))
        self.opt_show_avg_rating.setText(_("Show &average ratings"))
        self.opt_show_links_in_tag_browser.setToolTip(_("Show an icon if the item has an attached link"))
        self.opt_show_links_in_tag_browser.setText(_("Show &links icons"))
        self.opt_tag_browser_show_tooltips.setText(_("Show &tooltips"))
        self.opt_tag_browser_show_counts.setToolTip(_("<p>Show counts for items in the Tag browser. Such as the number of books\n"
"by each author, the number of authors, etc. If you turn it off, you can still\n"
"see the counts by hovering your mouse over any item.</p>"))
        self.opt_tag_browser_show_counts.setText(_("Show &counts"))
        self.opt_show_notes_in_tag_browser.setToolTip(_("Show an icon if the item has an attached note"))
        self.opt_show_notes_in_tag_browser.setText(_("Show &notes icons"))
        self.opt_tag_browser_old_look.setText(_("Use &alternating row colors"))
        self.opt_tag_browser_hide_empty_categories.setToolTip(_("<p>When checked, calibre will automatically hide any category\n"
"(a column, custom or standard) that has no items to show. For example, some\n"
"categories might not have values when using Virtual libraries. Checking this\n"
"box will cause these empty categories to be hidden.</p>"))
        self.opt_tag_browser_hide_empty_categories.setText(_("Hide empt&y categories (columns)"))
        self.opt_icons_on_right_in_tag_browser.setToolTip(_("If checked the notes and links icons will be placed at the right, after\n"
"the count and in columns. If unchecked, the icons will be placed immediately after the text,\n"
"to the left of the count and not in columns."))
        self.opt_icons_on_right_in_tag_browser.setText(_("Place icons on the &right, in columns"))
        self.opt_tag_browser_always_autocollapse.setToolTip(_("<p>When checked, Find in the Tag browser will show all items\n"
"that match the search instead of the first one. If Hide empty categories is\n"
"also checked then only categories containing a matched item will be shown.</p>"))
        self.opt_tag_browser_always_autocollapse.setText(_("Find &shows all items that match"))
        self.opt_tag_browser_allow_keyboard_focus.setToolTip(_("<p>When checked, the Tag browser can get keyboard focus, allowing\n"
"use of the keyboard to navigate the tree using the arrow keys. The Enter key simulates\n"
"a click on the selected item. The keyboard shortcut \'Tag browser /\n"
"Give the Tag browser keyboard focus\' changes the keyboard focus without\n"
"using the mouse.</p>"))
        self.opt_tag_browser_allow_keyboard_focus.setText(_("Allow the Tag browser to have keyboard &focus"))
        self.tb_focus_label.setText(_("<p style=\"text-indent: 2em; font-size:smaller\">\n"
"If you enable this option then you should set a keyboard shortcut to\n"
"focus the Tag browser under\n"
"<code>Preferences-&gt;Shortcuts-&gt;Tag browser-&gt;Give\n"
"the Tag browser keyboard focus</code></p>"))
        self.label1.setText(_("Spacing between &items:"))
        self.opt_tag_browser_item_padding.setToolTip(_("<p>The spacing between consecutive items in the Tag browser.\n"
"In units of (ex) which is the approximate height of the letter \'x\' in the\n"
"currently used font.</p>"))
        self.opt_tag_browser_item_padding.setSuffix(_("ex"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_tab1), _("Di&splay"))
        self.label_9.setText(_("&Category partitioning method:"))
        self.opt_tags_browser_partition_method.setToolTip(_("<p>Choose how Tag browser subcategories are displayed when\n"
"there are more items than the limit. Select by first\n"
"letter to see an A, B, C list. Choose partitioned to\n"
"have a list of fixed-sized groups. Set to disabled\n"
"if you never want subcategories</p>"))
        self.label_10.setText(_("Combine letters &when fewer items than:"))
        self.opt_tags_browser_collapse_fl_at.setToolTip(_("<p>If collapsing by first letter, combine adjacent letters together if\n"
"there are fewer items under a letter than specified here. If the partition method is\n"
"not set to first letter, this value is ignored. Set to zero to disable.</p>"))
        self.label_101.setText(_("Co&llapse when more items than:"))
        self.opt_tags_browser_collapse_at.setToolTip(_("<p>If a Tag browser category has more than this number of items, it is divided\n"
"up into subcategories. If the partition method is set to disable, this value is ignored.</p>"))
        self.label2.setToolTip(_("<p>Check the box for categories that are to\n"
"be partitioned using the criteria above. Uncheck the box if you don\'t want to\n"
"partition a category even if the number of items is larger than\n"
"the value shown above. This option can be used to\n"
"avoid collapsing hierarchical categories that have only\n"
"a few top-level elements.</p>"))
        self.label2.setText(_("Select categories to &partition:"))
        self.tb_partition_reset_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_partition_reset_button.setText(_("Reset list"))
        self.tb_partition_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same for each one.</p>"))
        self.tb_partition_import_layout_button.setText(_("Import list"))
        self.tb_partition_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same for each one.</p>"))
        self.tb_partition_export_layout_button.setText(_("Export list"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_tab2), _("&Partitioning and collapsing"))
        self.label3.setToolTip(_("<p>Check the box for an item if it is to be displayed as a\n"
"hierarchical tree in the Tag browser. For example, if you check\n"
"\'tags\' then tags of the form \'Mystery.English\'\n"
"and \'Mystery.Thriller\' will be displayed with English and Thriller\n"
"both under \'Mystery\'. If \'tags\' is not checked\n"
"then the tags will be displayed each on their own line.</p>\n"
"<p>The categories \'authors\', \'publisher\', \'news\', \'formats\', and \'rating\'\n"
"cannot be hierarchical.</p>"))
        self.label3.setText(_("Select categories with &hierarchical items:"))
        self.tb_hierarchy_reset_layout_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_hierarchy_reset_layout_button.setText(_("Reset list"))
        self.tb_hierarchy_import_layout_button.setToolTip(_("<p>Click this button to set the list to one\n"
"previously exported. This could be useful if you have several libraries with\n"
"similar structure and you want to use the same for each one.</p>"))
        self.tb_hierarchy_import_layout_button.setText(_("Import list"))
        self.tb_hierarchy_export_layout_button.setToolTip(_("<p>Click this button to write the current display\n"
"settings to a file. This could be useful if you have several libraries with similar\n"
"structure and you want to use the same for each one.</p>"))
        self.tb_hierarchy_export_layout_button.setText(_("Export list"))
        self.label4.setToolTip(_("<p>Set the order of the searches when clicking on an item in\n"
"the Tag browser. The \'or its children\' options are ignored when clicking on\n"
"top-level categories, items that aren\'t in a hierarchical category, and items\n"
"that don\'t have children.</p>"))
        self.label4.setText(_("Set the &order of searches when clicking on items"))
        self.tb_search_order_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.tb_search_order_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.tb_search_order_reset_button.setToolTip(_("Click this button to reset the list to its default order."))
        self.tb_search_order_reset_button.setText(_("Reset list"))
        self.tag_browser_tab.setTabText(self.tag_browser_tab.indexOf(self.tb_tab3), _("&Hierarchy and searching"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tag_browser_tab), _("&Tag browser"))
        self.opt_cb_fullscreen.setText(_("When showing in a separate window, show it &fullscreen"))
        self.opt_separate_cover_flow.setText(_("Show in a &separate window (needs restart)"))
        self.label_61.setText(_("&Number of covers to show in browse mode (needs restart):"))
        self.cover_browser_title_template_button.setText(_("Template &editor"))
        self.opt_cover_browser_title_template.setToolTip(_("<p>The template used to generate the text below the covers. This template uses\n"
"the same syntax as save templates. Defaults to just the book title.\n"
"Note that this setting is per-library, which means that you have to\n"
"set it again for every different calibre library you use. Use an\n"
"empty template for no text.</p>"))
        self.opt_cover_browser_reflections.setText(_("Show &reflections"))
        self.opt_cb_preserve_aspect_ratio.setToolTip(_("Show covers in their original aspect ratio instead of resizing\n"
"them to all have the same width and height"))
        self.opt_cb_preserve_aspect_ratio.setText(_("Preserve &aspect ratio of covers"))
        self.label_987.setText(_("Cover browser &position in the narrow layout:"))
        self.label_23.setText(_("&Template for caption:"))
        self.label_24.setText(_("Fie&ld for sub-title:"))
        self.fs_help_msg.setText(_("You can press the %s key to toggle full screen mode."))
        self.label_26.setText(_("Show ne&xt cover during auto scroll after:"))
        self.opt_books_autoscroll_time.setSuffix(_(" seconds"))
        self.opt_cb_double_click_to_activate.setText(_("&Double click to view the central book, instead of single click"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cover_browser_tab), _("Cover &browser"))
        self.opt_qv_respects_vls.setToolTip(_("<p>Check this box to make Quickview show books only in the\n"
"current Virtual library. If unchecked, Quickview ignores Virtual libraries. If\n"
"unchecked then only row changes are taken into account.</p>"))
        self.opt_qv_respects_vls.setText(_("&Apply Virtual libraries"))
        self.opt_qv_follows_column.setToolTip(_("<p>Check this box to make Quickview change the column being examined\n"
"when the column in the book list is changed using the cursor arrow keys</p>"))
        self.opt_qv_follows_column.setText(_("&Change Quickview item when book list column changes"))
        self.opt_qv_retkey_changes_column.setToolTip(_("<p>Pressing Enter in a cell changes both the book and the\n"
"column being examined (the left-hand panel)</p>"))
        self.opt_qv_retkey_changes_column.setText(_("&Pressing Enter changes the examined column"))
        self.opt_qv_dclick_changes_column.setToolTip(_("<p>Double-clicking in a cell changes both the book and the\n"
"column being examined (the left-hand panel)</p>"))
        self.opt_qv_dclick_changes_column.setText(_("&Double click changes examined column"))
        self.groupBox2.setTitle(_("Select columns to display"))
        self.qv_up_button.setToolTip(_("Move up. Keyboard shortcut: Ctrl-Up arrow"))
        self.qv_down_button.setToolTip(_("Move down. Keyboard shortcut: Ctrl-Down arrow"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.quickview_tab), _("&Quickview"))
from calibre.gui2.preferences import ListViewWithMoveByKeyPress, ListWidgetWithMoveByKeyPress
from calibre.gui2.widgets2 import ScrollingTabWidget



#!/usr/bin/env python


__license__ = 'GPL v3'
__copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
__docformat__ = 'restructuredtext en'

import datetime
import os
import time
from collections import namedtuple

from calibre import strftime
from calibre.customize import CatalogPlugin
from calibre.customize.conversion import DummyReporter, OptionRecommendation
from calibre.library import current_library_name
from calibre.library.catalogs import AuthorSortMismatchException, EmptyCatalogException
from calibre.ptempfile import PersistentTemporaryFile
from calibre.utils.localization import _, calibre_langcode_to_name, canonicalize_lang, get_lang

Option = namedtuple('Option', 'option, default, dest, action, help')


class EPUB_MOBI(CatalogPlugin):

    'EPUB catalog generator'

    name = 'Catalog_EPUB_MOBI'
    description = _('AZW3/EPUB/MOBI catalog generator')
    supported_platforms = ['windows', 'osx', 'linux']
    minimum_calibre_version = (0, 7, 40)
    author = 'Greg Riker'
    version = (1, 0, 0)
    file_types = {'azw3', 'epub', 'mobi'}

    THUMB_SMALLEST = "1.0"
    THUMB_LARGEST = "3.0"

    cli_options = [Option('--catalog-title',  # {{{
                          default='My Books',
                          dest='catalog_title',
                          action=None,
                          help=_('Title of generated catalog used as title in metadata.\n'
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--cross-reference-authors',
                          default=False,
                          dest='cross_reference_authors',
                          action='store_true',
                          help=_("Create cross-references in Authors section for books with multiple authors.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--debug-pipeline',
                           default=None,
                           dest='debug_pipeline',
                           action=None,
                           help=_("Save the output from different stages of the conversion "
                           "pipeline to the specified "
                           "folder. Useful if you are unsure at which stage "
                           "of the conversion process a bug is occurring.\n"
                           "Default: '%default'\n"
                           "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--exclude-genre',
                          default=r'\[.+\]|^\+$',
                          dest='exclude_genre',
                          action=None,
                          help=_("Regex describing tags to exclude as genres.\n"
                          "Default: '%default' excludes bracketed tags, e.g. '[Project Gutenberg]', and '+', the default tag for read books.\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--exclusion-rules',
                          default="(('Catalogs','Tags','Catalog'),)",
                          dest='exclusion_rules',
                          action=None,
                          help=_("Specifies the rules used to exclude books from the generated catalog.\n"
                          "The model for an exclusion rule is either\n('<rule name>','Tags','<comma-separated list of tags>') or\n"
                          "('<rule name>','<custom column>','<pattern>').\n"
                          "For example:\n"
                          "(('Archived books','#status','Archived'),)\n"
                          "will exclude a book with a value of 'Archived' in the custom column 'status'.\n"
                          "When multiple rules are defined, all rules will be applied.\n"
                          "Default: \n" + '"' + '%default' + '"' + "\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--generate-authors',
                          default=False,
                          dest='generate_authors',
                          action='store_true',
                          help=_("Include 'Authors' section in catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--generate-descriptions',
                          default=False,
                          dest='generate_descriptions',
                          action='store_true',
                          help=_("Include 'Descriptions' section in catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--generate-genres',
                          default=False,
                          dest='generate_genres',
                          action='store_true',
                          help=_("Include 'Genres' section in catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--generate-titles',
                          default=False,
                          dest='generate_titles',
                          action='store_true',
                          help=_("Include 'Titles' section in catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--generate-series',
                          default=False,
                          dest='generate_series',
                          action='store_true',
                          help=_("Include 'Series' section in catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--generate-recently-added',
                          default=False,
                          dest='generate_recently_added',
                          action='store_true',
                          help=_("Include 'Recently Added' section in catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--genre-source-field',
                          default=_('Tags'),
                          dest='genre_source_field',
                          action=None,
                          help=_("Source field for 'Genres' section.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--header-note-source-field',
                          default='',
                          dest='header_note_source_field',
                          action=None,
                          help=_("Custom field containing note text to insert in Description header.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--merge-comments-rule',
                          default='::',
                          dest='merge_comments_rule',
                          action=None,
                          help=_("#<custom field>:[before|after]:[True|False] specifying:\n"
                          " <custom field> Custom field containing notes to merge with comments\n"
                          " [before|after] Placement of notes with respect to comments\n"
                          " [True|False] - A horizontal rule is inserted between notes and comments\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--output-profile',
                          default=None,
                          dest='output_profile',
                          action=None,
                          help=_("Specifies the output profile. In some cases, an output profile is required to optimize"
                                 " the catalog for the device. For example, 'kindle' or 'kindle_dx' creates a structured"
                                 " Table of Contents with Sections and Articles.\n"
                                 "Default: '%default'\n"
                                 "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--prefix-rules',
                          default="(('Read books','tags','+','\u2713'),('Wishlist item','tags','Wishlist','\u00d7'))",
                          dest='prefix_rules',
                          action=None,
                          help=_("Specifies the rules used to include prefixes indicating read books, wishlist items and other user-specified prefixes.\n"
                          "The model for a prefix rule is ('<rule name>','<source field>','<pattern>','<prefix>').\n"
                          "When multiple rules are defined, the first matching rule will be used.\n"
                          "Default:\n" + '"' + '%default' + '"' + "\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--preset',
                          default=None,
                          dest='preset',
                          action=None,
                          help=_("Use a named preset created with the GUI catalog builder.\n"
                          "A preset specifies all settings for building a catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--use-existing-cover',
                          default=False,
                          dest='use_existing_cover',
                          action='store_true',
                          help=_("Replace existing cover when generating the catalog.\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                   Option('--thumb-width',
                          default='1.0',
                          dest='thumb_width',
                          action=None,
                          help=_("Size hint (in inches) for book covers in catalog.\n"
                          "Range: 1.0 - 2.0\n"
                          "Default: '%default'\n"
                          "Applies to: AZW3, EPUB, MOBI output formats")),
                          ]
    # }}}

    def run(self, path_to_output, opts, db, notification=DummyReporter()):
        from calibre.library.catalogs.epub_mobi_builder import CatalogBuilder
        from calibre.utils.config import JSONConfig
        from calibre.utils.logging import default_log as log

        # If preset specified from the cli, insert stored options from JSON file
        if hasattr(opts, 'preset') and opts.preset:
            available_presets = JSONConfig("catalog_presets")
            if opts.preset not in available_presets:
                if available_presets:
                    print(_('Error: Preset "%s" not found.' % opts.preset))
                    print(_('Stored presets: %s' % ', '.join([p for p in sorted(available_presets.keys())])))
                else:
                    print(_('Error: No stored presets.'))
                return 1

            # Copy the relevant preset values to the opts object
            for item in available_presets[opts.preset]:
                if item not in ['exclusion_rules_tw', 'format', 'prefix_rules_tw']:
                    setattr(opts, item, available_presets[opts.preset][item])

            # Provide an unconnected device
            opts.connected_device = {
                         'is_device_connected': False,
                         'kind': None,
                         'name': None,
                         'save_template': None,
                         'serial': None,
                         'storage': None,
                        }

            # Convert prefix_rules and exclusion_rules from JSON lists to tuples
            prs = []
            for rule in opts.prefix_rules:
                prs.append(tuple(rule))
            opts.prefix_rules = tuple(prs)

            ers = []
            for rule in opts.exclusion_rules:
                ers.append(tuple(rule))
            opts.exclusion_rules = tuple(ers)

        opts.log = log
        opts.fmt = self.fmt = path_to_output.rpartition('.')[2]

        # Add local options
        opts.creator = '{}, {} {}, {}'.format(strftime('%A'), strftime('%B'), strftime('%d').lstrip('0'), strftime('%Y'))
        opts.creator_sort_as = '{} {}'.format('calibre', strftime('%Y-%m-%d'))
        opts.connected_kindle = False

        # Finalize output_profile
        op = opts.output_profile
        if op is None:
            op = 'default'

        if opts.connected_device['name'] and 'kindle' in opts.connected_device['name'].lower():
            opts.connected_kindle = True
            if opts.connected_device['serial'] and \
               opts.connected_device['serial'][:4] in ['B004', 'B005']:
                op = "kindle_dx"
            else:
                op = "kindle"

        opts.description_clip = 380 if op.endswith('dx') or 'kindle' not in op else 100
        opts.author_clip = 100 if op.endswith('dx') or 'kindle' not in op else 60
        opts.output_profile = op

        opts.basename = "Catalog"
        opts.cli_environment = not hasattr(opts, 'sync')

        # Hard-wired to always sort descriptions by author, with series after non-series
        opts.sort_descriptions_by_author = True

        build_log = []

        build_log.append("%s('%s'): Generating %s %sin %s environment, locale: '%s'" %
            (self.name,
             current_library_name(),
             self.fmt,
             'for %s ' % opts.output_profile if opts.output_profile else '',
             'CLI' if opts.cli_environment else 'GUI',
             calibre_langcode_to_name(canonicalize_lang(get_lang()), localize=False))
             )

        # If exclude_genre is blank, assume user wants all tags as genres
        if opts.exclude_genre.strip() == '':
            # opts.exclude_genre = '\[^.\]'
            # build_log.append(" converting empty exclude_genre to '\[^.\]'")
            opts.exclude_genre = 'a^'
            build_log.append(" converting empty exclude_genre to 'a^'")
        if opts.connected_device['is_device_connected'] and \
           opts.connected_device['kind'] == 'device':
            if opts.connected_device['serial']:
                build_log.append(" connected_device: '%s' #%s%s " %
                    (opts.connected_device['name'],
                     opts.connected_device['serial'][0:4],
                     'x' * (len(opts.connected_device['serial']) - 4)))
                for storage in opts.connected_device['storage']:
                    if storage:
                        build_log.append("  mount point: %s" % storage)
            else:
                build_log.append(" connected_device: '%s'" % opts.connected_device['name'])
                try:
                    for storage in opts.connected_device['storage']:
                        if storage:
                            build_log.append("  mount point: %s" % storage)
                except:
                    build_log.append("  (no mount points)")
        else:
            build_log.append(" connected_device: '%s'" % opts.connected_device['name'])

        opts_dict = vars(opts)
        if opts_dict['ids']:
            build_log.append(" book count: %d" % len(opts_dict['ids']))

        sections_list = []
        if opts.generate_authors:
            sections_list.append('Authors')
        if opts.generate_titles:
            sections_list.append('Titles')
        if opts.generate_series:
            sections_list.append('Series')
        if opts.generate_genres:
            sections_list.append('Genres')
        if opts.generate_recently_added:
            sections_list.append('Recently Added')
        if opts.generate_descriptions:
            sections_list.append('Descriptions')

        if not sections_list:
            if opts.cli_environment:
                opts.log.warn('*** No Section switches specified, enabling all Sections ***')
                opts.generate_authors = True
                opts.generate_titles = True
                opts.generate_series = True
                opts.generate_genres = True
                opts.generate_recently_added = True
                opts.generate_descriptions = True
                sections_list = ['Authors', 'Titles', 'Series', 'Genres', 'Recently Added', 'Descriptions']
            else:
                opts.log.warn('\n*** No enabled Sections, terminating catalog generation ***')
                return ["No Included Sections", "No enabled Sections.\nCheck E-book options tab\n'Included sections'\n"]
        if opts.fmt == 'mobi' and sections_list == ['Descriptions']:
            warning = _("\n*** Adding 'By authors' section required for MOBI output ***")
            opts.log.warn(warning)
            sections_list.insert(0, 'Authors')
            opts.generate_authors = True

        opts.log(" Sections: %s" % ', '.join(sections_list))
        opts.section_list = sections_list

        # Limit thumb_width to 1.0" - 2.0"
        try:
            if float(opts.thumb_width) < float(self.THUMB_SMALLEST):
                log.warning(f"coercing thumb_width from '{opts.thumb_width}' to '{self.THUMB_SMALLEST}'")
                opts.thumb_width = self.THUMB_SMALLEST
            if float(opts.thumb_width) > float(self.THUMB_LARGEST):
                log.warning(f"coercing thumb_width from '{opts.thumb_width}' to '{self.THUMB_LARGEST}'")
                opts.thumb_width = self.THUMB_LARGEST
            opts.thumb_width = "%.2f" % float(opts.thumb_width)
        except Exception:
            log.error(f"coercing thumb_width from '{opts.thumb_width}' to '{self.THUMB_SMALLEST}'")
            opts.thumb_width = "1.0"

        # eval prefix_rules if passed from command line
        if type(opts.prefix_rules) is not tuple:
            try:
                opts.prefix_rules = eval(opts.prefix_rules)
            except:
                log.error("malformed --prefix-rules: %s" % opts.prefix_rules)
                raise
            for rule in opts.prefix_rules:
                if len(rule) != 4:
                    log.error("incorrect number of args for --prefix-rules: %s" % repr(rule))

        # eval exclusion_rules if passed from command line
        if type(opts.exclusion_rules) is not tuple:
            try:
                opts.exclusion_rules = eval(opts.exclusion_rules)
            except:
                log.error("malformed --exclusion-rules: %s" % opts.exclusion_rules)
                raise
            for rule in opts.exclusion_rules:
                if len(rule) != 3:
                    log.error("incorrect number of args for --exclusion-rules: %s" % repr(rule))

        # Display opts
        keys = sorted(opts_dict.keys())
        build_log.append(" opts:")
        for key in keys:
            if key in ['catalog_title', 'author_clip', 'connected_kindle', 'creator',
                       'cross_reference_authors', 'description_clip', 'exclude_book_marker',
                       'exclude_genre', 'exclude_tags', 'exclusion_rules', 'fmt',
                       'genre_source_field', 'header_note_source_field', 'merge_comments_rule',
                       'output_profile', 'prefix_rules', 'preset', 'read_book_marker',
                       'search_text', 'sort_by', 'sort_descriptions_by_author', 'sync',
                       'thumb_width', 'use_existing_cover', 'wishlist_tag']:
                build_log.append(f"  {key}: {repr(opts_dict[key])}")
        if opts.verbose:
            log('\n'.join(line for line in build_log))

        # Capture start_time
        opts.start_time = time.time()

        self.opts = opts

        if opts.verbose:
            log.info(" Begin catalog source generation (%s)" %
                     str(datetime.timedelta(seconds=int(time.time() - opts.start_time))))

        # Launch the Catalog builder
        catalog = CatalogBuilder(db, opts, self, report_progress=notification)

        try:
            catalog.build_sources()
            if opts.verbose:
                log.info(" Completed catalog source generation (%s)\n"  %
                         str(datetime.timedelta(seconds=int(time.time() - opts.start_time))))
        except (AuthorSortMismatchException, EmptyCatalogException) as e:
            log.error(" *** Terminated catalog generation: %s ***" % e)
        except:
            log.error(" unhandled exception in catalog generator")
            raise

        else:
            recommendations = []
            recommendations.append(('remove_fake_margins', False,
                OptionRecommendation.HIGH))
            recommendations.append(('comments', '', OptionRecommendation.HIGH))

            """
            >>> Use to debug generated catalog code before pipeline conversion <<<
            """
            GENERATE_DEBUG_EPUB = False
            if GENERATE_DEBUG_EPUB:
                catalog_debug_path = os.path.join(os.path.expanduser('~'), 'Desktop', 'Catalog debug')
                setattr(opts, 'debug_pipeline', os.path.expanduser(catalog_debug_path))

            dp = getattr(opts, 'debug_pipeline', None)
            if dp is not None:
                recommendations.append(('debug_pipeline', dp,
                    OptionRecommendation.HIGH))

            if opts.output_profile and opts.output_profile.startswith("kindle"):
                recommendations.append(('output_profile', opts.output_profile,
                    OptionRecommendation.HIGH))
                recommendations.append(('book_producer', opts.output_profile,
                    OptionRecommendation.HIGH))
                if opts.fmt == 'mobi':
                    recommendations.append(('no_inline_toc', True,
                        OptionRecommendation.HIGH))
                    recommendations.append(('verbose', 2,
                        OptionRecommendation.HIGH))

            # Use existing cover or generate new cover
            cpath = None
            existing_cover = False
            try:
                search_text = 'title:"{}" author:{}'.format(
                        opts.catalog_title.replace('"', '\\"'), 'calibre')
                matches = db.search(search_text, return_matches=True, sort_results=False)
                if matches:
                    cpath = db.cover(matches[0], index_is_id=True, as_path=True)
                    if cpath and os.path.exists(cpath):
                        existing_cover = True
            except:
                pass

            if self.opts.use_existing_cover and not existing_cover:
                log.warning("no existing catalog cover found")

            if self.opts.use_existing_cover and existing_cover:
                recommendations.append(('cover', cpath, OptionRecommendation.HIGH))
                log.info("using existing catalog cover")
            else:
                from calibre.ebooks.covers import calibre_cover2
                log.info("replacing catalog cover")
                new_cover_path = PersistentTemporaryFile(suffix='.jpg')
                new_cover = calibre_cover2(opts.catalog_title, 'calibre')
                new_cover_path.write(new_cover)
                new_cover_path.close()
                recommendations.append(('cover', new_cover_path.name, OptionRecommendation.HIGH))

            # Run ebook-convert
            from calibre.ebooks.conversion.plumber import Plumber
            plumber = Plumber(os.path.join(catalog.catalog_path, opts.basename + '.opf'),
                            path_to_output, log, report_progress=notification,
                            abort_after_input_dump=False)
            plumber.merge_ui_recommendations(recommendations)
            plumber.run()

            try:
                os.remove(cpath)
            except:
                pass

            if GENERATE_DEBUG_EPUB:
                from calibre.ebooks.epub import initialize_container
                from calibre.ebooks.tweak import zip_rebuilder
                from calibre.utils.zipfile import ZipFile
                input_path = os.path.join(catalog_debug_path, 'input')
                epub_shell = os.path.join(catalog_debug_path, 'epub_shell.zip')
                initialize_container(epub_shell, opf_name='content.opf')
                with ZipFile(epub_shell, 'r') as zf:
                    zf.extractall(path=input_path)
                os.remove(epub_shell)
                zip_rebuilder(input_path, os.path.join(catalog_debug_path, 'input.epub'))

            if opts.verbose:
                log.info(" Catalog creation complete (%s)\n" %
                     str(datetime.timedelta(seconds=int(time.time() - opts.start_time))))

        # returns to gui2.actions.catalog:catalog_generated()
        return catalog.error



#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import absolute_import, division, print_function, unicode_literals

__license__   = 'GPL v3'
__copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
__docformat__ = 'restructuredtext en'

import time
from io import StringIO
from threading import Event, Thread

from calibre.customize.ui import metadata_plugins
from calibre.ebooks.metadata.sources.base import create_log
from calibre.ebooks.metadata.sources.prefs import msprefs
from calibre.utils.img import image_from_data, image_to_data, remove_borders_from_image, save_cover_data_to
from calibre.utils.imghdr import identify
from polyglot.queue import Empty, Queue


class Worker(Thread):

    def __init__(self, plugin, abort, title, authors, identifiers, timeout, rq, get_best_cover=False):
        Thread.__init__(self)
        self.daemon = True

        self.plugin = plugin
        self.abort = abort
        self.get_best_cover = get_best_cover
        self.buf = StringIO()
        self.log = create_log(self.buf)
        self.title, self.authors, self.identifiers = (title, authors,
                identifiers)
        self.timeout, self.rq = timeout, rq
        self.time_spent = None

    def run(self):
        start_time = time.time()
        if not self.abort.is_set():
            try:
                if self.plugin.can_get_multiple_covers:
                    self.plugin.download_cover(self.log, self.rq, self.abort,
                        title=self.title, authors=self.authors, get_best_cover=self.get_best_cover,
                        identifiers=self.identifiers, timeout=self.timeout)
                else:
                    self.plugin.download_cover(self.log, self.rq, self.abort,
                        title=self.title, authors=self.authors,
                        identifiers=self.identifiers, timeout=self.timeout)
            except:
                self.log.exception('Failed to download cover from',
                        self.plugin.name)
        self.time_spent = time.time() - start_time


def is_worker_alive(workers):
    for w in workers:
        if w.is_alive():
            return True
    return False


def process_result(log, result):
    plugin, data = result
    try:
        if getattr(plugin, 'auto_trim_covers', False):
            img = image_from_data(data)
            nimg = remove_borders_from_image(img)
            if nimg is not img:
                data = image_to_data(nimg)
        fmt, width, height = identify(data)
        if width < 0 or height < 0:
            raise ValueError('Could not read cover image dimensions')
        if width < 50 or height < 50:
            raise ValueError('Image too small')
        data = save_cover_data_to(data)
    except Exception:
        log.exception('Invalid cover from', plugin.name)
        return None
    return (plugin, width, height, fmt, data)


def run_download(log, results, abort,
        title=None, authors=None, identifiers={}, timeout=30, get_best_cover=False):
    '''
    Run the cover download, putting results into the queue :param:`results`.

    Each result is a tuple of the form:

        (plugin, width, height, fmt, bytes)

    '''
    if title == _('Unknown'):
        title = None
    if authors == [_('Unknown')]:
        authors = None

    plugins = [p for p in metadata_plugins(['cover']) if p.is_configured()]

    rq = Queue()
    workers = [Worker(p, abort, title, authors, identifiers, timeout, rq, get_best_cover=get_best_cover) for p
            in plugins]
    for w in workers:
        w.start()

    first_result_at = None
    wait_time = msprefs['wait_after_first_cover_result']
    found_results = {}

    start_time = time.time()  # Use a global timeout to workaround misbehaving plugins that hang
    while time.time() - start_time < 301:
        time.sleep(0.1)
        try:
            x = rq.get_nowait()
            result = process_result(log, x)
            if result is not None:
                results.put(result)
                found_results[result[0]] = result
                if first_result_at is not None:
                    first_result_at = time.time()
        except Empty:
            pass

        if not is_worker_alive(workers):
            break

        if first_result_at is not None and time.time() - first_result_at > wait_time:
            log('Not waiting for any more results')
            abort.set()

        if abort.is_set():
            break

    while True:
        try:
            x = rq.get_nowait()
            result = process_result(log, x)
            if result is not None:
                results.put(result)
                found_results[result[0]] = result
        except Empty:
            break

    for w in workers:
        wlog = w.buf.getvalue().strip()
        log('\n'+'*'*30, w.plugin.name, 'Covers', '*'*30)
        log('Request extra headers:', w.plugin.browser.addheaders)
        if w.plugin in found_results:
            result = found_results[w.plugin]
            log('Downloaded cover:', '%dx%d'%(result[1], result[2]))
        else:
            log('Failed to download valid cover')
        if w.time_spent is None:
            log('Download aborted')
        else:
            log('Took', w.time_spent, 'seconds')
        if wlog:
            log(wlog)
        log('\n'+'*'*80)


def download_cover(log,
        title=None, authors=None, identifiers={}, timeout=30):
    '''
    Synchronous cover download. Returns the "best" cover as per user
    prefs/cover resolution.

    Returned cover is a tuple: (plugin, width, height, fmt, data)

    Returns None if no cover is found.
    '''
    rq = Queue()
    abort = Event()

    run_download(log, rq, abort, title=title, authors=authors,
            identifiers=identifiers, timeout=timeout, get_best_cover=True)

    results = []

    while True:
        try:
            results.append(rq.get_nowait())
        except Empty:
            break

    cp = msprefs['cover_priorities']

    def keygen(result):
        plugin, width, height, fmt, data = result
        return (cp.get(plugin.name, 1), 1/(width*height))

    results.sort(key=keygen)

    return results[0] if results else None
